/*
 * smp2ul - convert simple multiple precision number to unsigned
 */

#ifndef lint
static char _cpyrgt[] = "Copyright 1989 Howard Lee Gayle";
#endif lint

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 1,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <howard/port.h>
#include <howard/version.h>

MODVER ("@(#)$Header: smp2u.c,v 1.2 89/08/14 18:29:32 howard Exp $");

#include <errno.h>
#include <limits.h>
#include <howard/registers.i>
#include <howard/simultipre.i>
#include <howard/smp.h>

PUBLIC int smp2u (sp, lp)
R1 smpT     *sp; /* Points to SMP.*/
R5 unsigned *lp; /* Points to where to store result.*/

/* Function:
 *    
 * Algorithm:
 *    Step downward through higher order SMP elements that ought to
 *    be zero, and make sure they are.  Make sure high order bits in
 *    the highest order element are also zero.  Or each element into
 *    the result.  Check for a negative result.  Store the result.
 * Returns:
 *    
 * Notes:
 *    
 */
{
R4 unsigned res = 0; /* Result.*/
R2 smpElT  *p1;      /* Steps through sp->smp_el[].*/
R3 smpElT  *p2;      /* Highest order smp_el used.*/

if (((smpT *) NULL) == sp) return (EINVAL);
p2 = &sp->smp_el[SMPNU - 1];
for (p1 = &sp->smp_el[SMPNEL - 1]; p1 != p2; --p1)
   if (0 != *p1) return (ERANGE);
if (0 != (*p1 >> SMPUHB)) return (ERANGE);
while (p1 >= sp->smp_el)
   res = (res << SMPEB) | *p1--;
if (sp->smp_neg && (0 != res)) return (ERANGE);
if (((unsigned *) NULL) != lp) *lp = res;
return (SUCCESS);
}

#ifdef TEST
#include <howard/usage.h>

MAINVER ("@(#)$Header: smp2u.c,v 1.2 89/08/14 18:29:32 howard Exp $");
USAGE ("");

#include <howard/malf.h>

PRIVATE void t (num, neg, hi, lo, xrc, xres)
int   num; /* Test number.*/
boolT neg; /* smp_neg */
smpElT hi; /* smp_el[1] */
smpElT lo; /* smp_el[0] */
int   xrc; /* Expected return code.*/
unsigned xres; /* Expected result.*/
{
int rc; /* Return code.*/
unsigned res; /* Result stored here.*/
smpT smp; /* SMP argument.*/

smp.smp_neg = neg;
smp.smp_el[0] = lo;
smp.smp_el[1] = hi;
rc = smp2u (&smp, &res);
if (rc != xrc) PRINTF ("%d: rc %d expected %d\n", num, rc, xrc);
if ((0 == rc) && (0 == xrc) && (res != xres))
   PRINTF ("%d: res %X expected %X\n", num, res, xres);
}

PUBLIC int main ()
{
/* num    sgn   hi      lo      rc         res   */
t(__LINE__, 0,      0,      0,      0, (unsigned)           0);
t(__LINE__, 1,      0,      0,      0, (unsigned)           0);
t(__LINE__, 0,      0,      1,      0, (unsigned)           1);
t(__LINE__, 0, 0x7FFF, 0xFFFF,      0, (unsigned)  0x7FFFFFFF);
t(__LINE__, 0, 0x8000,      0,      0, (unsigned)  0x80000000);
t(__LINE__, 0, 0xFFFF, 0xFFFF,      0, (unsigned)  0xFFFFFFFF);
t(__LINE__, 0,0x10000, 0x0000, ERANGE, (unsigned)           0);
t(__LINE__, 1,      0,      1, ERANGE, (unsigned)           0);

mfflush (stdout, "Standard Output");
exit (SUCCESS);

#ifdef lint
return (SUCCESS);
#endif
}
#endif
